/* Copyright IBM Corp. 2010  All Rights Reserved. */
/* 
 * @author Ben
 * Summary: Generic iWidget that displays a visual vertical menu from context menu data
 */
dojo.provide("com.ibm.mm.builder.systemwidget.ContentSetMenu");

dojo.require("com.ibm.pb.widgets.ContentMenu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit.MenuSeparator");
dojo.require("com.ibm.pb.iwidget.iWidgetBase");

/**
 * Generic iWidget that renders a visual menu that represents data retrieved from the public
 * com.ibm.mashups.builder.model.ContextMenu APIs.
 * 
 * Event-based, so it opens when a certain event is fired, and closes on another event.
 * Builds its contents dynamically at open time based on the payload's resourceId and the
 * widget's resourceType.
 *  
 */
dojo.declare("com.ibm.mm.builder.systemwidget.ContentSetMenu", [com.ibm.pb.iwidget.iWidgetBase], {

	/**
	 * iEvent that this widget listens to to open its menu when fired; requires JSON payload including
	 * a domNode property referencing where the menu should open, and a resourceId property referencing
	 * what resource ID the menu uses when getting its data from a context menu
	 */
	openEvent: "",
	
	/**
	 * iEvent that this widget listens to to close its menu when fired
	 */
	closeEvent: "",
	
	/**
	 * String CSS class to apply to the menu root node 
	 */
	menuCSSClass: "",
	
	/**
	 * String CSS class to apply to the node where the menu is attached when the menu
	 * is open, and removed when the menu is closed.  Is not necessarily an anchor element 
	 * regardless of the name of the property
	 */
	anchorCSSClass: "",
	
	/**
	 * Array of DOM nodes or DOM node IDs that this menu will open for onClick 
	 */
	//domNodes: [],
	
	/**
	 * Actual rich UI Menu widget
	 */
	menu:null,
	
	/**
	 * The ID of the context menu that this module gets its data from
	 */
	contextMenuID: "",
	
	/**
	 * The type of the resource that this module uses when getting its data from a context menu
	 */
	resourceType: "",
	
	updated:false,

    /**
     * A javascript method to be executed once before the menu is opened. This will only be called before the menu is constructed initially, not for subsequent opens.
     */
    beforeOpen: null,
	
	constructor: function() {
		this._menuConnections = [];
		this._submenus = [];
	},

	onLoad: function() {
		var mname = "onLoad", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		this.inherited(arguments);
        dojo.mixin(this, this.getItemSetValues(["contextMenuID","resourceType","openEvent", "closeEvent", "menuCSSClass","anchorCSSClass","beforeOpen"]));
		this.id = this.iContext.widgetId;
		// Do we support fixing the menu to domNodes based on its itemSet attributes?
		/*
		if(dojo.isString(this.domNodes)) {
			this.domNodes = this.domNodes.split(",");
		}
		else if(!dojo.isArray(this.domNodes)) {
			this.domNodes = [this.domNodes];
		}
		*/
		if(this.openEvent) {
			lgr.trace(mname, "Subscribing to open event ${0}", [this.openEvent]);
			this.evtSvc.subscribeEvent(this.openEvent, this, this.open, null, this.id);
		}
		if(this.closeEvent) {
			lgr.trace(mname, "Subscribing to close event ${0}", [this.closeEvent]);
			this.evtSvc.subscribeEvent(this.closeEvent, this, this.close, null, this.id);
		}
//		this._initMenu();
		com.ibm.mashups.builder.model.Factory.getContextMenuModel().registerListener(this.contextMenuID, this, "_menuChanged");
		lgr.exiting(mname, arguments);
	},
	
	_menuChanged:function(){
		this.updated=true;
	},
	
	open: function(payload) {
		var mname = "open", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		try {
			if(!this.menu ||this.updated || this.menu.resourceID != payload.resourceId) {
				this._initMenu();
                if (this.beforeOpen) {
                    try {
                        dojo.eval(this.beforeOpen+"()");
                    }catch(ex){
                        console.log("Error when calling beforeOpen: ", ex);
                    }
                }
				// The current menu contents weren't built from the payload.resourceId, so rebuild them
				this.menu.initContents(payload.resourceId);
			}
			var me = this, ori = payload.orientation;
			if(!ori) {
				ori = this.menu.isLeftToRight() ? 
					{'BL':'TL','TL':'BL','BR':'TR','TR':'BR'} : 
					{'BR':'TR','TR':'BR','BL':'TL','TL':'BL'};
			}
			var menu = this.menu;
			var oldFocus,closer;
			
			try{
				oldFocus = payload.domNode || dijit.getFocus(menu);
				closer = function() {
					if(menu.refocus) {
						dijit.focus(oldFocus);
					}
					me.close();
				};	
			}catch(ex){
				console.log("error when getFocus");
			}

			lgr.trace(mname, "Popping up menu ${0} around ${1} using orientation ${2}", [this.menu, payload.domNode, ori]);
			if(payload.domNode && this.anchorCSSClass) {
				dojo.addClass(payload.domNode, this.anchorCSSClass);
			}
			dijit.popup.open({
				popup: this.menu,
				orient: ori,
				around: payload.domNode,
				onExecute: closer,
				onCancel: closer,
				onClose: function() {
					if(payload.domNode && me.anchorCSSClass) {
						dojo.removeClass(payload.domNode, me.anchorCSSClass);
					}
				}
			});
            // this if statement is to get around a dojo bug.
            if (this.menu.focusedChild) {
                if (this.menu.focusedChild._destroyed) {
                    this.menu.focusedChild = null;
                }
            }
			this.menu.focus();
			this.updated=false;
		}
		catch(err) {
			lgr.severe(mname, "Error opening menu ${0} ${1}", [this, err]);
			throw err;
		}
		lgr.exiting(mname, arguments);
	},
	
	close: function(payload) {
		var mname = "close", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		if(this.menu.close) {
			this.menu.close();
		}
		else {
			dijit.popup.close(this.menu);
		}
		lgr.exiting(mname, arguments);
	},
	_initMenu: function(isSubMenu) {
		var mname = "_initMenu", lgr = this.LOGGER, me = this;
		lgr.entering(mname, arguments);
		// destroy and recreate the menu
		while(this._menuConnections.length > 0) {
			dojo.disconnect(this._menuConnections.shift());
		}
		if(this.menu) {
            try {
                this.menu.destroyRecursive();
            }
            catch (e) {
                // in recent drivers this crashed on IE because dojo query got an exception.
                // This is a tactical solution because of regression phase.
                // A final solution has to be looked at.
            }
		}
		var initArgs = {
			contextMenuID: this.contextMenuID,
			resourceType: this.resourceType,
			leftClickToOpen: true,
			offset_y: 9,
			internalRefresh:false
		};
		if(this.menuCSSClass) {
			initArgs["class"] = this.menuCSSClass;
			initArgs.getInheritableParams = function(){
				var args = this.inherited("getInheritableParams", arguments);
				args["class"] = me.menuCSSClass;
				return args;
			};
		}
		this.menu = new com.ibm.pb.widgets.ContentMenu(initArgs);
		this._menuConnections.push(dojo.connect(this.menu, "onBlur", this, this.close));
		this.menu.startup();

        if (document.body.dir=='rtl' && dojo.isSafari && navigator.userAgent.indexOf('iPad') != -1) {
            dojo.removeClass(this.menu.domNode,"dijitMenuTable");
        }

		lgr.exiting(mname, arguments);
	},
	onUnload: function(){
		if(this.menu) {
			this.menu.destroyRecursive();
			delete this.menu;
		}
	}
});
